unit acdesrtu;
// Sorts procedure blocks within a Delphi unit into alphabetical order
// Adds a documentation block on the bottom (ie after "end.")
//        +--------------------------------------------------------------+
//        |  Documentation standard used                                 |
//        +--------------------------------------------------------------+
//        |  The procedure declaration and any lines following it        |
//        |  up to the first of var,const or begin are reported          |
//        |                                                              |
//        |  eg  Procedure Foo;                          ) THESE LINES   |
//        |      { This does something exciting! }       ) APPEAR IN     |
//        |      // Too good to be true!                 ) DOCUMENTATION |
//        |      begin                                                   |
//        |         beep;                                                |
//        |      end;                                                    |
//        +--------------------------------------------------------------+
// Written by Peter Fox
// peter@eminent.demon.co.uk
// Do whatever you like with it at your own risk
// 22 December 98
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, FileCtrl;

type
  TForm1 = class(TForm)
    FL: TFileListBox;
    DL: TDirectoryListBox;
    busort: TButton;
    DriveComboBox1: TDriveComboBox;
    Label1: TLabel;
    lastatus: TLabel;
    procedure busortClick(Sender: TObject);
    procedure FLChange(Sender: TObject);
  private
    procedure Resort(const f:string);
    function Startscodeblock(const s:string):boolean;
    { Private declarations }
  public
    { Public declarations }
  end;

function  SortProcs(Item1, Item2: Pointer): Integer;
procedure MakeBackupFile(const filename:string);


var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.busortClick(Sender: TObject);
{run through list of files sorting each selecetd in turn}
var
  i : integer;
  f : string;
begin
  for i := 0 to Fl.items.count - 1 do begin
    if fl.selected[i] then begin
      f := fl.items[i];
      makebackupfile(f);
      resort(f);
      fl.selected[i] := false;    // remove selection
    end;
  end;
end;


procedure tform1.Resort(const f:string);
{resort the procedures in a single source code file}
var
  // L : LIST    ---------- Why use a TLIST ---------------------------
  // when we're in the implementation section of the code we cut out
  // each procedure into a TStringlist of its own then shuffle these
  // around prior to writing them out again.
  // We might have declared an array of TStringList but this is unlikely
  // to be correctly sized.  A List is just as easy to use and comes
  // ready made with a sort method where all we have to do is provide
  // a function that compares two list elements.  Easy!
  //
  // L := TList.create;
  // TRY  {protect list resource}
  //   L.ADD(<something we can point to>);
  //   L.<other methods lists have>;
  //   MyObjInstance := TMyObj(L[i]);
  //   L.Sort(<your custom comparison routine here>);
  // FINALLY
  //   For i := 0 to L.Count-1 do              // free up all the objects
  //     if L[i] <> nil then L[i].free;        // in the list ...
  //   L.free;                                 // ... and the list itself
  // END;

  fin,fout : Textfile;
  foutn,finn : string;
  L : TList;
  s : string;
  li,i,j : integer;
begin

  // get our files sorted out
  foutn := f;
  finn := changefileext(foutn,'.bak');
  assignfile(Fin,finn);
  assignfile(Fout,foutn);
  reset(fin);
  rewrite(fout);

  // various preparations
  L := tlist.create;
  li := -1;

  lastatus.caption := 'Sorting '+foutn;
  application.processmessages;

  TRY //PROTECTING TLIST RESOURCE

    // read the interface section - make no changes here
    while not eof(fin) do begin
      readln(fin,s);
      writeln(fout,s);
      if trim(s) = 'implementation' then break;
    end;


    // now if we see procedure or function etc we know this is the start of code
    while not eof(fin) do begin
      readln(fin,s);
      if startscodeblock(s) then li := L.add(TStringlist.create);
      if li < 0 then begin
        writeln(fout,s);              // bit before first procedure
      end else begin
        if length(s)>3 then begin     // test for "end." ie end of code
          if 'end.'= lowercase(copy(trim(s),1,4)) then break;
        end;
        TStringlist(L.items[li]).add(s);
      end;
    end;

    // now sort the list by first line of the procedure name
    L.sort(sortprocs);


    // now write out our strings by running through the list
    for i := 0 to l.count - 1 do begin
      writeln(fout,'');
      writeln(fout,'');
      writeln(fout,'');
      writeln(fout,'');
      with Tstringlist(L[i]) do begin
        // remove any blank lines from the end of this block
        j := count-1;
        while j >= 0 do begin
          if trim(strings[j])<>'' then break;
          delete(j);
          dec(j);
        end;
        // write the code out
        for j := 0 to count-1 do writeln(fout,strings[j]);
      end;
    end;
    writeln(fout,'');               // terminate source
    writeln(fout,'end.');


    // now write out a documentation block
    // Standard is to report {...} following the function definition
    // before var/const/begin
    for i := 0 to 5 do writeln(fout,'');
    writeln(fout,'(*  Usage comments extracted on '+formatdatetime('c',now));
    for i := 0 to 5 do writeln(fout,'************************************************************');
    for i := 0 to l.count - 1 do begin
      writeln(fout,'');
      writeln(fout,'---------------------------------------------------------------------------');
      with Tstringlist(L[i]) do begin
        for j := 0 to count-1 do begin
          s := strings[j];
          if pos('var',trim(s))=1 then break;
          if pos('begin',trim(s))=1 then break;
          if pos('const',trim(s))=1 then break;
          writeln(fout,s);
        end;
      end;
    end;
    writeln(fout,'');               // terminate source
    writeln(fout,'*)');

  FINALLY  // close files and free list etc
    closefile(fout);
    closefile(fin);
    // free the resources used by all our TStringlists
    for i := 0 to l.count -1 do Tstringlist(l[i]).free;
    // now free the list itself
    L.free;
  END;
  lastatus.caption := foutn+' sorted';
  application.processmessages;
end;

function Tform1.StartsCodeBlock(const s:string):boolean;
{true if a procedure of some sort}
var
  t : string;
begin
  result := false;
  if length(s)<8 then exit;
  t := trim(s);
  if pos(lowercase(copy(t,1,1)),'pfcd')=0 then exit;
  if pos(lowercase(copy(t,2,1)),'ruoe')=0 then exit;
  t := lowercase(copy(t,1,11));
  result :=  (pos('procedure',t)=1)
           or(pos('function',t)=1)
           or(pos('constructor',t)=1)
           or(pos('descructor',t)=1);
end;


function  SortProcs(Item1, Item2: Pointer): Integer;
{ order procedures by name }
// NB This sort procedure can't be TForm1.... I forget the reason but
// take it from me it needs to be at the unit level
var
  s1,s2 : string;
  i : integer;
begin
  // the first line will be the function name etc
  s1 := trim(TStringlist(item1).strings[0]);
  s2 := trim(TStringlist(item2).strings[0]);
  // lose the function/procedure bit etc
  i := pos(' ',s1);
  s1 := trim(copy(s1,i+1,99));
  i := pos(' ',s2);
  s2 := trim(copy(s2,i+1,99));
  result := ansicomparetext(s1,s2);
end;


procedure TForm1.FLChange(Sender: TObject);
begin
  busort.enabled := (FL.selcount>0);
end;

procedure MakeBackupFile(const filename:string);
{ rename the specified file ...BAK
  deals with previous b/u versions
  and handles current not yet existing}
var
  ok : boolean;
  f : string;
begin
  ok := true;
  if fileexists(filename) then begin
    f := changefileext(filename,'.bak');
    if fileexists(f) then ok := deletefile(f);
    ok := ok and renamefile(Filename,f);
    if not ok then showmessage('Trouble making a backup for '+filename);
  end;
end;


end.

